package com.huimv.eartag2.process2.listener;

import cn.hutool.core.text.csv.CsvUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.huimv.eartag2.common.mq.Const;
import com.huimv.eartag2.process2.service.IDeviceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Component
@Slf4j
public class DeviceListener {
    @Autowired
    private RedisTemplate redisTemplate;
    @Autowired
    private IDeviceService deviceService;

    /**
     * @Method      : processDeviceHeartbeat
     * @Description : 处理设备心跳数据
     * @Params      : [RawMap]
     * @Return      : void
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/2/15       
     * @Time        : 8:56
     */
    @RabbitListener(queues = Const.QUEUE_DEVICE_HEARTBEAT)
    @RabbitHandler
    public void processDeviceHeartbeat(Map DeviceMap) throws ParseException {
        System.out.println("222 <<<<<<<<<<<<<<< 处理设备心跳数据>>"+DeviceMap.toString());
        //--更新设备注册数据(MySQL,Redis)-
        //--保存设备在线统计数据(MySQL,Redis)-
        //--保存设备状态数据(MySQL,Redis)-
        JSONObject deviceJo = JSON.parseObject(DeviceMap.get("heartbeat").toString());
        String deviceCode = deviceJo.getString("device");
        System.out.println("deviceCode>>"+deviceCode);
        //--设备在线统计表
        //--设备状态每日统计表
        //--设备和耳标总状态表

//        test1();

        //# 心跳数加1，更改在线统计数据表（MySQL），更新Redis; #
        deviceService.updateDeviceRegister(deviceCode);

        //# 心跳数加1，更改在线统计数据表（MySQL），更新Redis; #
        deviceService.updateDeviceOnline(deviceCode);

        //# 处理状态数据 #
        deviceService.updateFarmDeviceStatus(deviceCode);
    }

    private void test1() {
            String deviceCode = "202007239999869";
//            Map map = new HashMap<>();
//            map.put("name","栋舍");
//            map.put("id","001");
//            System.out.println("map>>"+map.toString());
            String deviceRegHashKey = "device_register_"+deviceCode;
            //为hash结构设置多个键值对(hmset)
//        redisTemplate.opsForHash().putAll(deviceRegHashKey, map);


//            Object actObj = redisTemplate.opsForHash().get(deviceCode, "act");
//            System.out.println("actObj>>"+actObj);
        Object nameObj = redisTemplate.opsForHash().get(deviceRegHashKey,"name");
        System.out.println("nameObj>>"+nameObj.toString());
    }

    /**
     * @Method      : processDeviceTemp
     * @Description : 处理设备温度数据
     * @Params      : [RawMap]
     * @Return      : void
     *
     * @Author      : ZhuoNing
     * @Date        : 2022/2/15
     * @Time        : 9:05
     */
    @RabbitListener(queues = Const.QUEUE_DEVICE_TEMP)
    @RabbitHandler
    public void processDeviceTemp(Map DeviceTempMap) throws ParseException {
        System.out.println("333 <<<<<<<<<<<<<<< 处理设备温度数据>>"+DeviceTempMap.toString());
        JSONObject deviceJo = JSON.parseObject(DeviceTempMap.get("deviceTemp").toString());
        String deviceCode = deviceJo.getString("device");
        System.out.println("333 设备编号>>"+deviceCode);
        //--保存设备注册数据(MySQL,Redis)
        //--保存设备状态数据(MySQL,Redis)

        //--设备在线统计表
        //--设备状态每日统计表
        //--设备和耳标总状态表

//        test1();
        //# 处理状态数据 #
        deviceService.updateFarmDeviceStatus(deviceCode);

    }
}
